<?php
/*
 * Plugin Name: Deny All Firewall
 * Version: 1.5.5
 * Plugin URI: https://webd.uk/deny-all-firewall/
 * Description: Blocks access to everything except genuine site content using .htaccess
 * Author: Webd Ltd
 * Author URI: https://webd.uk
 * Text Domain: deny-all-firewall
 */



if (!defined('ABSPATH')) {
    exit('This isn\'t the page you\'re looking for. Move along, move along.');
}



if (!class_exists('daf_class')) {

	class daf_class {

        public static $version = '1.5.5';

        private $black_list;

		function __construct() {

            register_deactivation_hook(__FILE__, array($this, 'daf_deactivation'));
	        add_action('admin_menu', array($this, 'daf_settings_menu'));
            $this->black_list = array(
                '/comments/feed/',
                '/feed/',
                '/wp-includes/wlwmanifest.xml',
                '/wp-json/',
                '/wp-json/oembed/1.0/embed',
                '/xmlrpc.php',
                '/'
            );
            add_action('wp_login', array($this, 'daf_refresh_htaccess_on_login'), 10, 2);
            add_action('update_option_daf_options', array($this, 'daf_refresh_htaccess_on_update_options'), 10, 2);
            add_action('add_option_daf_options', array($this, 'daf_refresh_htaccess_on_add_options'), 10, 2);
            add_action('save_post', array($this, 'daf_content_changed'), 10, 3);
            add_action('edited_term', array($this, 'daf_content_changed'), 10, 3);
            add_action('created_term', array($this, 'daf_content_changed'), 10, 3);
            add_action('attachment_updated', array($this, 'daf_content_changed'), 10, 3);
            add_action('add_attachment', array($this, 'daf_content_changed'));
            add_action('after_switch_theme', array($this, 'daf_content_changed'), 10, 2);
            add_action('load-post.php', array($this, 'daf_theme_options_metabox_setup'));
            add_action('load-post-new.php', array($this, 'daf_theme_options_metabox_setup'));

            $options = get_option('daf_options');

            if (isset($options['enable_firewall']) && $options['enable_firewall'] && get_option('daf_content_changed')) {

                add_action('admin_notices', array($this, 'daf_content_changed_admin_notice'));

            }

            add_action('daf_refresh', array($this, 'daf_refresh_htaccess_hourly_if_changed_and_firewall_enabled'));

            if (!wp_next_scheduled('daf_refresh')) {

                wp_schedule_event(time()+3600, 'hourly', 'daf_refresh');

            }

            if (is_admin()) {

                add_action('wp_ajax_daf_refresh_rules', array($this, 'daf_ajax_refresh_rules'));
                add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'daf_add_plugin_settings_links'));

            }

            add_filter('site_status_tests', array($this, 'daf_site_status_tests'));

		}

		function daf_add_plugin_settings_links($links) {

			$settings_links = dafCommon::plugin_action_links(add_query_arg('page', 'daf_settings', admin_url('options-general.php')));

			return array_merge($settings_links, $links);

		}

        function daf_content_changed($item_id, $second = false, $third = false) {

            if (!get_option('daf_content_changed')) {

                $options = get_option('daf_options');

                if (!isset($options['allow_all_content']) && current_action() == 'save_post' && $second instanceof WP_Post && isset($second->post_status) && in_array($second->post_status, array('inherit', 'publish', 'private'))) {

                    if (get_post_types(array('name' => $second->post_type, 'exclude_from_search' => false))) {

                        if (!$this->daf_is_permalink_in_htaccess(parse_url(get_permalink($item_id)))) {

                            update_option('daf_content_changed', true);

                        }

                    }

                    return;

                } elseif (!isset($options['allow_all_content']) && current_action() == 'edited_term') {

                    if (!$this->daf_is_permalink_in_htaccess(parse_url(get_term_link($item_id)))) {

                        update_option('daf_content_changed', true);

                    }

                    return;

                } elseif (!isset($options['allow_all_content']) && current_action() == 'created_term') {

                    if (!$this->daf_is_permalink_in_htaccess(parse_url(get_term_link($item_id)))) {

                        update_option('daf_content_changed', true);

                    }

                    return;

                } elseif (!isset($options['allow_all_content']) && current_action() == 'attachment_updated' && $second instanceof WP_Post && isset($second->post_status) && in_array($second->post_status, array('inherit', 'publish', 'private'))) {

                    if (!$this->daf_is_permalink_in_htaccess(parse_url(get_permalink($second->ID)))) {

                        update_option('daf_content_changed', true);

                    }

                    return;

                } elseif (!isset($options['allow_all_content']) && current_action() == 'add_attachment') {

                    $second = get_post($item_id);

                    if (isset($second->post_title)) {

                        $pathinfo = pathinfo($second->post_title);

                    }

                    if (isset($pathinfo['extension'])) {

                        $extension = $pathinfo['extension'];

                    } else {

                        $extension = false;

                    }

                    if (isset($second->post_status) && !($second->post_status == 'private' && $extension == 'zip') && in_array($second->post_status, array('inherit', 'publish', 'private')) && !$this->daf_is_permalink_in_htaccess(parse_url(get_permalink($second->ID)))) {

                        update_option('daf_content_changed', true);

                    }

                    return;

                } elseif (current_action() == 'after_switch_theme') {

                    update_option('daf_content_changed', true);

                    return;

                }

            }

        }

        function daf_is_permalink_in_htaccess($permalink) {

            if (isset($permalink['port'])) {

                $reconstructed_host = $permalink['host'] . ':' . $permalink['port'];

            } else {

                $reconstructed_host = $permalink['host'];

            }

            if ($reconstructed_host == $_SERVER['HTTP_HOST'] && $permalink['path'] !== '/') {

                $current_htaccess = file_get_contents(dafCommon::get_home_path() . '.htaccess');

                if (strpos($current_htaccess, 'RewriteCond %{REQUEST_URI} "!^' . urldecode($permalink['path']) . '$') !== false) {

                    return true;

                }

            } elseif ($permalink['path'] === '/') {

                return true;

            }

            return false;

        }

        function daf_content_changed_admin_notice() {

?>

<div class="notice notice-warning daf-content-changed-notice">

<p><strong>Content Changed</strong><br />
Your site's content has changed. You will need to refresh the firewall rules to allow new content through the firewall ...</p>
<p><a href="#" id="daf-refresh-rules" title="<?= __('Refresh Firewall Rulees', 'deny-all-firewall'); ?>" class="button-primary">Refresh Firewall Rules</a></p>

</div>

<script>
    jQuery(document).on('click', '#daf-refresh-rules', function() {
        if (confirm(<?= json_encode(__('Are you sure?', 'deny-all-firewall')); ?>)) {
            jQuery('#daf-refresh-rules').after(' <img src="/wp-admin/images/loading.gif" style="vertical-align: bottom;" />');
            jQuery.ajax({
            	url: ajaxurl,
            	data: {
                	action: 'daf_refresh_rules',
                	_ajax_nonce: '<?= wp_create_nonce('daf-refresh-rules'); ?>'
            	},
            	success: function(result){
            	    if (result.success) {
                        jQuery('.daf-content-changed-notice').fadeTo(100, 0, function() {
                            jQuery('.daf-content-changed-notice').slideUp(100, function() {
                                jQuery('.daf-content-changed-notice').remove();
                            });
                        });
            	    } else {
                        alert('Something went wrong, sorry!');
                        jQuery('.daf-content-changed-notice img').remove();
            	    }
                },
            	error: function(){
                    alert('Something went wrong, sorry!');
                    jQuery('.daf-content-changed-notice img').remove();
                }
        	});
        }
    });
</script>

<?php

        }

        function daf_ajax_refresh_rules() {

            check_ajax_referer('daf-refresh-rules');

            if (current_user_can('manage_options')) {

                $daf_htaccess = $this->daf_create_htaccess();

                if ($daf_htaccess) {

                    if ($this->daf_remove_rules()) {

                        $this->daf_inject_rules($daf_htaccess);

                        wp_send_json_success();

                    } else {

                        wp_send_json_error();

                    }

            	} else {

                    wp_send_json_error();

            	}

            } else {

                wp_send_json_error();

            }

        	wp_die();

        }

		function daf_settings_menu() {

            if (is_admin()) {

    		    add_options_page('Deny All Firewall Settings','Deny All Firewall', 'manage_options', 'daf_settings', array($this, 'daf_settings_page'));

	            add_action('admin_init', array($this, 'daf_register_settings'));

            }

		}

		function daf_settings_page() {

?>
<div class="wrap">
<div>
<h2><?= __('Deny All Firewall', 'deny-all-firewall'); ?></h2>

<?php

                $options = get_option('daf_options');
                $external_ip = get_option('daf_external_ip');
    			$current_htaccess = file_get_contents(dafCommon::get_home_path() . '.htaccess');

                if (strpos($current_htaccess, '# BEGIN Deny All Firewall') !== false && !isset($options['enable_firewall'])) {

                    $options['enable_firewall'] = true;
                    update_option('daf_options', $options);

                } elseif (strpos($current_htaccess, '# BEGIN Deny All Firewall') === false && isset($options['enable_firewall'])) {

                    unset($options['enable_firewall']);
                    update_option('daf_options', $options);

                }

                if (isset($options['enable_firewall']) && $options['enable_firewall'] && !$external_ip) {
?>

<p><strong style="color: red;">Your server's external IP address could not be detected so the plugin was unable to create rules to block access to /wp-admin/.</strong></p>

<?php

                }

?>

<form action="options.php" method="post">
<?php

            settings_fields('daf_options');
            do_settings_sections('daf_settings_sections');

?>

<p class="submit"><input type="submit" name="submit" id="submit" class="button button-primary" value="<?php _e('Save Changes','deny-all-firewall'); ?>"></p>

</form></div>

<?php

            if (!isset($options['enable_log'])) {

                if (file_exists(dirname(dirname(__DIR__)).'/ENABLE_403_LOG')) { unlink(dirname(dirname(__DIR__)).'/ENABLE_403_LOG'); }
                if (file_exists(dirname(dirname(__DIR__)).'/403.log')) { unlink(dirname(dirname(__DIR__)).'/403.log'); }

            } else {

                if (!file_exists(dirname(dirname(__DIR__)).'/ENABLE_403_LOG')) { file_put_contents(dirname(dirname(__DIR__)).'/ENABLE_403_LOG', '', FILE_APPEND); }

                if (file_exists(dirname(dirname(__DIR__)).'/403.log')) {

                    $log403 = explode("\n", implode('', file(dirname(dirname(__DIR__)).'/403.log')));
                    $log_stats = array();
                    $count_log_entries = 0;
                    $log_started = false;

                    foreach ($log403 as $log_entry) {

                        if (isset($log_entry[0]) && $log_entry[0] == '{' && substr($log_entry, -1) == '}') {

                            $log_entry_object = json_decode($log_entry);

                            if (isset($log_entry_object->time) && isset($log_entry_object->remote_addr) && isset($log_entry_object->redirect_url)) {

                                $count_log_entries++;

                                if (!$log_started) { $log_started = $log_entry_object->time; }

                                if ((time() - $log_entry_object->time) < 86401) {


                                    $log_stats[] = array(
                                        'time' => $log_entry_object->time,
                                        'remote_addr' => $log_entry_object->remote_addr,
                                        'redirect_url' => $log_entry_object->redirect_url
                                    );

                                }

                            }

                        }

                    }

                    $last_hour_stats = array_count_values(array_column($log_stats, 'redirect_url'));
                    arsort($last_hour_stats);
                    $top_ten_stats = array_slice($last_hour_stats, 0, 50)

?>

<h3>Top 50 Blocked Requests in the last 24 Hours</h3>

<p>Your logfile contains details of <strong><?= $count_log_entries; ?></strong> blocked requests since <?= date_i18n(get_option('date_format') . ' ' . get_option('time_format'), $log_started); ?>.</p>

<p>That's <strong><?= absint($count_log_entries / ((time() - $log_started) / 3600)) ?> requests</strong> blocked per hour!

<p>Here are the top fifty in the last 24 hours ...</p>

<table class="wp-list-table widefat striped">
<thead>
<tr>
<td class="manage-column column-cb check-column"></td>
<th class="manage-column column-name column-primary">Blocked Request</th>
<th class="manage-column column-total-requests">Total Requests</th>
<th class="manage-column column-description">Possible Source</th>
</tr>
</thead>
<tbody>
<?php

                    $i = 0;

                    foreach ($top_ten_stats as $key => $value) {

                        $i++;
                        $key = htmlentities(urldecode($key));
                        $request_type = $this->daf_check_blocked_request($key);
?>

<tr>
<td class="check-column"><?= $i; ?>)</td>
<td class="plugin-title column-primary"><?= $key; ?> <?php

                        if (!preg_match('/\/(.*)\.(.*)\//', $key) && file_exists($_SERVER['DOCUMENT_ROOT'] . $key)) {

                            if (preg_match('/\/$/', $key)) {

                                echo '<span style="color: orange;">(directory exists)</span> ';

                            } else {

                                echo '<span style="color: orange;">(file exists)</span> ';

                            }

                        }

?><span class="daf-unblock button button-small" data-request="<?= esc_html($key); ?>"><?= __('Unblock', 'deny-all-firewall'); ?></span></td>
<td class="column-total-requests"><?= $value; ?></td>
<td class="column-description"><?php

                        if (is_array($request_type)) {

                            if ($request_type['suspicious']) { echo '<span style="color: red;">'; }

                            echo $request_type['description'];

                            if ($request_type['suspicious']) { echo '</span>'; }

                        }

?></td>
</tr>

<?php

                    }

?>
</tbody>
</table>

<script type="text/javascript">
jQuery('.daf-unblock').click(function() {
    jQuery('.whitelist').last().val(jQuery(this).data('request'));
    alert(jQuery(this).data('request') + ' added to whitelist. Don\'t forget to "Save Changes".');
});
</script>

<?php

                } else {

?>

<h3>Log file is currently empty</h3>

<?php

                }

            }
?>
</div>
<?php
		}

        function daf_check_blocked_request($request) {

            $types_of_request = array(
                array(
                    'pattern' => '/\/robots.txt$/',
                    'description' => 'Web crawler looking to see if it is allowed to index your website in its search engine. Wordpress creates a generic robots.txt file but doing so loads the entire Wordpress engine and uses up resources on your server so unless you have your own robots.txt file, this should remain blocked.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/\/xmlrpc.php$/',
                    'description' => 'xmlrpc.php allows users to publish posts remotely but it has been abused in the past so if you don\'t use it we recommend that it remains blocked.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/\/wp-includes\/wlwmanifest.xml$/',
                    'description' => 'This file tells Windows Live Writer how to talk to wordpress. If you don\'t use Windows Live Writer then we don\'t see why this file shouldn\'t be blocked.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/\/wp-admin\/options.php$/',
                    'description' => 'If you are trying to change the Administration Email Address on your website you will need to deactivate the firewall in order to do so.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/ads.txt$/',
                    'description' => 'Web crawler trying to find your authorised digital sellers. Unless you have your own ads.txt file, this should remain blocked.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/favicon.ico$/',
                    'description' => 'Visitor\'s browser asking for the website\'s icon. If you haven\'t set your own custom icon, the Wordpress default one will be loaded, so this can remain blocked.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/(admin|login)/',
                    'description' => 'Hacker looking for your login page. Once found they will then try to brute force your login details to gain access to your site.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/.(php|php[0-9]{1}|asp|aspx)$/',
                    'description' => 'Hacker scanning your site for malicious or vulnerable code that they can use to compromise your website.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/.php.suspected$/',
                    'description' => 'Hacker scanning your site for malicious code that has already been marked as suspicious.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/\/wp-content\/plugins\/(.*)\/(readme.txt|license.txt|(.*).css)$/i',
                    'description' => 'Hacker scanning your site for a particular plugin to see if you have a version that can be exploited.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/\/wp-content\/themes\/(.*)\/style.css$/',
                    'description' => 'Hacker scanning your site for a particular theme to see if you have a version that can be exploited.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/config/',
                    'description' => 'Hacker trying to find your site\'s configuration settings which will allow them to gain access to your database.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/apple-touch-icon(.*).png$/',
                    'description' => 'Visitor\'s Apple device looking for an icon. If you haven\'t set your own custom Apple icons then this can remain blocked.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/\/feed\/$/',
                    'description' => 'RSS feed.',
                    'suspicious' => false
                ),
                array(
                    'pattern' => '/.sql/',
                    'description' => 'Hacker trying to download a backup of your database. With this they can extract your user details and hashed passwords.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/.(zip|gz)$/',
                    'description' => 'Hacker trying to download an archive of your website or database which will allow them to gain access to your site.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/\/.user.ini$/',
                    'description' => 'Hacker trying to view your .user.ini file.',
                    'suspicious' => true
                ),
                array(
                    'pattern' => '/\/wp-json\/wp\/v2\/users\/[0-9]+$/',
                    'description' => 'Hacker trying to scan your site for usernames.',
                    'suspicious' => true
                )
            );

            foreach ($types_of_request as $type_of_request) {

                if (preg_match($type_of_request['pattern'], $request)) {

                    return array(
                        'description' => $type_of_request['description'],
                        'suspicious' => $type_of_request['suspicious']
                    );
                    break;

                }

            }

            return false;

        }

        function daf_register_settings() {

            $options = get_option('daf_options');

        	register_setting(
        	    'daf_options',
        	    'daf_options',
        	    array($this, 'daf_options_validate')
            );
            add_settings_section(
                'daf_firewall_options',
                __('Firewall Options','deny-all-firewall'),
                array($this, 'daf_firewall_options_text'),
                'daf_settings_sections'
            );
            add_settings_field(
                'enable_firewall',
                __('Enable Firewall','deny-all-firewall'),
                array($this, 'daf_enable_firewall_string'),
                'daf_settings_sections',
                'daf_firewall_options'
            );
            add_settings_field(
                'enable_lock_down',
                __('Enable Lock Down','deny-all-firewall'),
                array($this, 'daf_enable_lock_down_string'),
                'daf_settings_sections',
                'daf_firewall_options'
            );
            add_settings_field(
                'allow_all_content',
                __('Allow All Content','deny-all-firewall'),
                array($this, 'daf_allow_all_content_string'),
                'daf_settings_sections',
                'daf_firewall_options'
            );
            add_settings_field(
                'allow_all_ips',
                __('Allow All IPs','deny-all-firewall'),
                array($this, 'daf_allow_all_ips_string'),
                'daf_settings_sections',
                'daf_firewall_options'
            );
            add_settings_field(
                'force_ssl',
                __('Force SSL','deny-all-firewall'),
                array($this, 'daf_force_ssl_string'),
                'daf_settings_sections',
                'daf_firewall_options'
            );
            add_settings_field(
                'enable_log',
                __('Enable Log','deny-all-firewall'),
                array($this, 'daf_enable_log_string'),
                'daf_settings_sections',
                'daf_firewall_options'
            );
            add_settings_section(
                'daf_whitelisted_requests',
                __('Whitelisted Requests','deny-all-firewall'),
                array($this, 'daf_whitelisted_requests_text'),
                'daf_settings_sections'
            );

            $whitelist_id = 1;

            while (isset($options['whitelist_' . $whitelist_id])) {

                add_settings_field(
                    'whitelist_' . $whitelist_id,
                    sprintf(__('Whitelisted Request', 'deny-all-firewall'), $whitelist_id),
                    array($this, 'daf_whitelist_string'),
                    'daf_settings_sections',
                    'daf_whitelisted_requests',
                    array('whitelist_id' => $whitelist_id)
                );
               $whitelist_id++;

            }

            add_settings_field(
                'whitelist_' . $whitelist_id,
                sprintf(__('Add Request', 'deny-all-firewall'), $whitelist_id),
                array($this, 'daf_whitelist_string'),
                'daf_settings_sections',
                'daf_whitelisted_requests',
                array('whitelist_id' => $whitelist_id)
            );
            add_settings_section(
                'daf_sitemap_options',
                __('Sitemap Options','deny-all-firewall'),
                array($this, 'daf_sitemap_options_text'),
                'daf_settings_sections'
            );
            add_settings_field(
                'enable_sitemap',
                __('Enable Sitemap','deny-all-firewall'),
                array($this, 'daf_enable_sitemap_string'),
                'daf_settings_sections',
                'daf_sitemap_options'
            );

        }

        function daf_firewall_options_text() {

?>
<p><?php _e('Use these settings to configure the firewall. You can save these settings again to quickly allow new content through the firewall.','deny-all-firewall'); ?></p>
<?php

        }

        function daf_enable_firewall_string() {

            $options = get_option('daf_options');

?>
<input id="enable_firewall" name="daf_options[enable_firewall]" type="radio" value=""<?php checked('1', (!isset($options['enable_firewall']))); ?> /> Firewall Disabled<br />
<input id="enable_firewall" name="daf_options[enable_firewall]" type="radio" value="1"<?php checked('1', (isset($options['enable_firewall']) && dafCommon::sanitize_boolean($options['enable_firewall']))); ?> /> Firewall Enabled
<?php

        }

        function daf_enable_lock_down_string() {

            $options = get_option('daf_options');

?>
<input id="enable_lock_down" name="daf_options[enable_lock_down]" type="checkbox" value="1"<?php checked('1', (isset($options['enable_lock_down']) && dafCommon::sanitize_boolean($options['enable_lock_down']))); ?> />
Block all Query Strings and POST data<br />(strips querystrings and blocks POST data before they reach Wordpress so may prevent some site features from working)
<?php

        }

        function daf_allow_all_content_string() {

            $options = get_option('daf_options');

?>
<input id="allow_all_content" name="daf_options[allow_all_content]" type="checkbox" value="1"<?php checked('1', (isset($options['allow_all_content']) && dafCommon::sanitize_boolean($options['allow_all_content']))); ?> />
Allows requests for all content (posts, pages, tags, categories, etc) even if it does not exist<br />(not secure but required if your site has a lot of content)<br /><strong>This option will disable the Sitemap feature</strong> (default Wordpress sitemap will be enabled)
<?php

        }

        function daf_allow_all_ips_string() {

            $options = get_option('daf_options');

?>
<input id="allow_all_ips" name="daf_options[allow_all_ips]" type="checkbox" value="1"<?php checked('1', (isset($options['allow_all_ips']) && dafCommon::sanitize_boolean($options['allow_all_ips']))); ?> />
Does not lock /wp-admin/ down to authenticated users<br />(not secure but required if your site has a lot of users)
<?php

        }

        function daf_force_ssl_string() {

            $options = get_option('daf_options');

?>
<input id="force_ssl" name="daf_options[force_ssl]" type="checkbox" value="1"<?php checked('1', (isset($options['force_ssl']) && dafCommon::sanitize_boolean($options['force_ssl']))); ?> />
If you use an SSL certificate, this setting will force visitors to use HTTPS rather than HTTP
<?php

        }

        function daf_enable_log_string() {

            $options = get_option('daf_options');

?>
<input id="enable_log" name="daf_options[enable_log]" type="checkbox" value="1"<?php checked('1', (isset($options['enable_log']) && dafCommon::sanitize_boolean($options['enable_log']))); ?> />
Enable logging of blocked requests<br />(only turn this on when required, a large log file will eventually slow down this settings page considerably)
<?php

        }

        function daf_whitelisted_requests_text() {

?>
<p><?php _e('If something is being blocked that shouldn\'t be, you can unblock it manually here. You can also create 301 redirects.','deny-all-firewall'); ?></p>
<?php

        }

        function daf_whitelist_string($args) {

            $options = get_option('daf_options');

            if (isset($options['whitelist_' . $args['whitelist_id']])) {

?>
<span class="dashicons dashicons-trash"></span><input type="checkbox" onclick="jQuery('#whitelist_<?= $args['whitelist_id']; ?>').prop('disabled', function(i, v) { return !v; });" />
<?php

            }

?>
<input id="whitelist_<?= $args['whitelist_id']; ?>" class="whitelist" name="daf_options[whitelist_<?= $args['whitelist_id']; ?>]" type="text" value="<?= ((isset($options['whitelist_' . $args['whitelist_id']])) ? sanitize_text_field($options['whitelist_' . $args['whitelist_id']]) : ''); ?>" placeholder="/hello-world/" />
301 Redirect <input type="checkbox" onclick="jQuery('#whitelist_<?= $args['whitelist_id']; ?>_301').val(''); jQuery('#whitelist_<?= $args['whitelist_id']; ?>_301').prop('disabled', function(i, v) { return !v; }); jQuery('#whitelist_<?= $args['whitelist_id']; ?>_301_wrapper').toggle();"<?php checked(isset($options['whitelist_' . $args['whitelist_id'] . '_301']), true); ?> />
<span id="whitelist_<?= $args['whitelist_id']; ?>_301_wrapper"<?= ((!isset($options['whitelist_' . $args['whitelist_id'] . '_301'])) ? ' style="display: none;"' : ''); ?>>to <input id='whitelist_<?= $args['whitelist_id']; ?>_301' name='daf_options[whitelist_<?= $args['whitelist_id']; ?>_301]' type='text' value='<?= ((isset($options['whitelist_' . $args['whitelist_id'] . '_301'])) ? sanitize_text_field($options['whitelist_' . $args['whitelist_id'] . '_301']) : ''); ?>'<?= ((!isset($options['whitelist_' . $args['whitelist_id'] . '_301'])) ? ' disabled="disabled"' : ''); ?> placeholder="https://www.hello.com/world/" /></span>
<?php

        }

        function daf_sitemap_options_text() {

?>
<p><?php _e('This plugin can automatically generate an XML sitemap with much richer content than the default WordPress generated one and will then let search engines know where to find it with robots.txt. Not available when "Allow All Content" has been selected.','deny-all-firewall'); ?></p>
<?php

        }

        function daf_enable_sitemap_string() {

            $options = get_option('daf_options');

            $disabled = false;

            if (defined('WPSEO_VERSION')) {

                $yoast_options = get_option('wpseo');

                if (isset($yoast_options['enable_xml_sitemap']) && $yoast_options['enable_xml_sitemap']) {

                    $disabled = true;

?>
<p style="color: red"><strong>Yoast SEO is installed so you'll need to <a href="<?= add_query_arg( 'page', 'wpseo_dashboard', admin_url('admin.php#top#features') ); ?>">turn off "XML sitemaps" in the settings</a> to use this sitemap instead.</p>
<?php

                }

            }

?>
<input id="enable_sitemap" name="daf_options[enable_sitemap]" type="radio" value=""<?php checked('1', (!isset($options['enable_sitemap']))); ?><?= ($disabled ? ' disabled' : ''); ?> /> Sitemap Disabled<br />
<input id="enable_sitemap" name="daf_options[enable_sitemap]" type="radio" value="1"<?php checked('1', (isset($options['enable_sitemap']) && dafCommon::sanitize_boolean($options['enable_sitemap']))); ?><?= ($disabled ? ' disabled' : ''); ?> /> Sitemap Enabled
<?php

        }

        function daf_options_validate($input) {

            $options = get_option('daf_options');

            if (isset($input['enable_firewall']) && $input['enable_firewall']) {

        	    $options['enable_firewall'] = true;

            } else {

                $this->daf_remove_rules();

                if (isset($options['enable_firewall'])) {

                    unset($options['enable_firewall']);

                }

            }

            if (isset($input['enable_lock_down'])) {

        	    $options['enable_lock_down'] = dafCommon::sanitize_boolean($input['enable_lock_down']);

            } elseif (isset($options['enable_lock_down'])) {

                unset($options['enable_lock_down']);

            }

            if (isset($input['allow_all_content'])) {

        	    $options['allow_all_content'] = dafCommon::sanitize_boolean($input['allow_all_content']);

            } elseif (isset($options['allow_all_content'])) {

                unset($options['allow_all_content']);

            }

            if (!isset($options['allow_all_content']) && isset($input['enable_sitemap']) && $input['enable_sitemap']) {

        	    $options['enable_sitemap'] = true;

            } else {

                $this->daf_remove_sitemap();

                if (isset($options['enable_sitemap'])) {

                    unset($options['enable_sitemap']);

                }

            }

            if (isset($input['allow_all_ips'])) {

        	    $options['allow_all_ips'] = dafCommon::sanitize_boolean($input['allow_all_ips']);

            } elseif (isset($options['allow_all_ips'])) {

                unset($options['allow_all_ips']);

            }

            if (isset($input['force_ssl'])) {

        	    $options['force_ssl'] = dafCommon::sanitize_boolean($input['force_ssl']);

            } elseif (isset($options['force_ssl'])) {

                unset($options['force_ssl']);

            }

            if (isset($input['enable_log'])) {

        	    $options['enable_log'] = dafCommon::sanitize_boolean($input['enable_log']);

            } elseif (isset($options['enable_log'])) {

                unset($options['enable_log']);

            }

            $whitelist_input = array();

            foreach ($input as $key => $value) {

                if ((substr($key, 0, strlen('whitelist_')) === 'whitelist_') && !(substr($key, -strlen('_301')) === '_301')) {

                    if (sanitize_text_field($value) != '') {

                        if (isset($input[$key . '_301']) && sanitize_text_field($input[$key . '_301']) != '') {

                            $whitelist_input[] = array('uri' => sanitize_text_field($value), 'redirect' => sanitize_text_field($input[$key . '_301']));

                        } else {

                            $whitelist_input[] = array('uri' => sanitize_text_field($value));

                        }

                    }

                }

            }

            usort($whitelist_input, array($this, 'daf_compare_uri'));

            foreach ($options as $key => $value) {

                if (substr($key, 0, strlen('whitelist_')) === 'whitelist_') {

                    unset($options[$key]);

                }

            }

            $whitelist_id = 1;

            foreach ($whitelist_input as $whitelist_array) {

                $options['whitelist_' . $whitelist_id] = $whitelist_array['uri'];

                if (isset($whitelist_array['redirect'])) {

                    $options['whitelist_' . $whitelist_id . '_301'] = $whitelist_array['redirect'];

                }

                $whitelist_id++;

            }

            if ($options == get_option('daf_options') && isset($options['enable_firewall']) && $options['enable_firewall']) {

                $this->daf_refresh_htaccess_on_update_options(false, $options);

            }

            if (isset($options['whitelist'])) {

                unset($options['whitelist']);

            }

            if (isset($options['content_changed_notices'])) {

        	    unset($options['content_changed_notices']);

            }

            if (isset($options['enable_auto_update'])) {

                unset($options['enable_auto_update']);

            }

            return $options;

        }

        function daf_compare_uri($a, $b) {

            return strnatcmp($a['uri'], $b['uri']);

        }

		function daf_remove_rules() {

			if (is_writable(dafCommon::get_home_path() . '.htaccess')) {

                copy(dafCommon::get_home_path() . '.htaccess', dafCommon::get_home_path() . '.htaccess_bak');

				$htaccess_rules = explode("\n", implode('', file(dafCommon::get_home_path() . '.htaccess')));
				$found = false;
				$blank_line = false;
				$newdata = '';

				foreach ($htaccess_rules as $htaccess_rule) {

						if ($htaccess_rule == '# BEGIN Deny All Firewall') {

							$found = true;

						}

						if (!$found) {

							$newdata .= "$htaccess_rule\n";

						}

						if ($htaccess_rule == '# END Deny All Firewall') {

							$found = false;

						}

				}

                $newdata = trim($newdata,"\n");

				$f = @fopen(dafCommon::get_home_path() . '.htaccess', 'w');
				fwrite($f, $newdata);

                return true;

			}

            return false;

		}

        function daf_remove_sitemap() {

			if (is_writable(dafCommon::get_home_path() . 'sitemap.xml')) {

                unlink(dafCommon::get_home_path() . 'sitemap.xml');

			}

			if (is_writable(dafCommon::get_home_path() . 'robots.txt')) {

                unlink(dafCommon::get_home_path() . 'robots.txt');

			}

        }

		function daf_create_htaccess($current_user_id = false) {

            $options = get_option('daf_options');
            if (!$current_user_id) { $current_user_id = get_current_user_id(); }
            $external_ip = get_option('daf_external_ip');

            if (!$external_ip) {

                $external_ip_response = wp_remote_get(plugins_url('remote-addr.php', __FILE__), array('timeout' => 10, 'sslverify' => false));

                if (!is_wp_error($external_ip_response) && isset($external_ip_response['response']['code']) && $external_ip_response['response']['code'] == 200) {

                    $external_ip = wp_remote_retrieve_body($external_ip_response);

                    if (!empty($external_ip)) {

                        $external_ip_parts = explode('.', $external_ip);

                        if (count($external_ip_parts) > 3) {

                            $external_ip = $external_ip_parts[0].'.'.$external_ip_parts[1].'.'.$external_ip_parts[2].'.';

                            update_option('daf_external_ip', $external_ip);

                        } else {

                            $external_ip = false;

                        }

                    } else {

                        $external_ip = false;

                    }

                }

            }

            $session_ips = array();

            if (!isset($options['allow_all_ips']) && $external_ip) {

                if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                    update_user_meta($current_user_id, 'daf_REMOTE_ADDR', $_SERVER['HTTP_CF_CONNECTING_IP']);

                } elseif (isset($_SERVER['REMOTE_ADDR'])) {

                    update_user_meta($current_user_id, 'daf_REMOTE_ADDR', $_SERVER['REMOTE_ADDR']);

                }

                $logged_in_users = get_users(
                    array(
                        'meta_key' => 'session_tokens'
                    )
                );

                foreach ($logged_in_users as $logged_in_user) {

                    $session_tokens = get_user_meta($logged_in_user->ID, 'session_tokens', true);

                    foreach ($session_tokens as $session_token) {

                        $session_ips[] = $session_token['ip'];

                    }

                    if (get_user_meta($logged_in_user->ID, 'daf_REMOTE_ADDR', true)) {

                        $session_ips[] = get_user_meta($logged_in_user->ID, 'daf_REMOTE_ADDR', true);

                    }

                }

                $session_ips = array_unique($session_ips);
                sort($session_ips);

            }

            $http_url = ((isset($_SERVER['HTTPS']) ? "https" : "http") . '://' . $_SERVER['HTTP_HOST']);

            $detected_urls = array();

            $whitelisted_requests = array();
            $whitelisted_redirects = array();
            $whitelist_id = 1;

            while (isset($options['whitelist_' . $whitelist_id])) {

                if (isset($options['whitelist_' . $whitelist_id . '_301'])) {

                    $whitelisted_redirects[] = array(
                        'request' => $options['whitelist_' . $whitelist_id],
                        'destination' => $options['whitelist_' . $whitelist_id . '_301']
                    );

                } else {

                    $whitelisted_requests[] = $options['whitelist_' . $whitelist_id];
                }

                $whitelist_id++;

            }

            $all_post_types = get_post_types(array('exclude_from_search' => false));

            if (!isset($options['allow_all_content'])) {

                $all_posts = new WP_Query(array(
                    'post_type'      => 'any',
                    'post_status'    => array('inherit', 'publish', 'private'),
                    'posts_per_page' => -1
                ));

                foreach ($all_posts->posts as $post) {

                    if (in_array($post->post_type, $all_post_types) && comments_open($post)) {

                        $parsed_url = parse_url(site_url('wp-comments-post.php'));

                        if (!in_array($parsed_url['path'], $whitelisted_requests)) {

                            $whitelisted_requests[] = $parsed_url['path'];

                        }

                    }

            		if (class_exists('woocommerce') && $post->post_type == 'product') {
    		    
                        $parsed_url = parse_url(get_permalink($post));

                        if (!in_array($parsed_url['path'], $whitelisted_requests)) {

                            $whitelisted_requests[] = $parsed_url['path'];

                        }

            		} else {

                        $detected_urls[] = get_permalink($post);

            		}

                    if (isset($options['enable_sitemap']) && $options['enable_sitemap'] && $post->post_status != 'private') {

                        if ($post->post_type == 'page') {

                            $sitemap_urls[] = array(
                                'loc' => get_permalink($post),
                                'lastmod' => get_the_modified_date('Y-m-d',$post),
                                'changefreq' => 'weekly',
                                'priority' => '1.0'
                            );

                        } else {

                            $sitemap_urls[] = array(
                                'loc' => get_permalink($post),
                                'lastmod' => get_the_modified_date('Y-m-d',$post),
                                'changefreq' => 'weekly',
                                'priority' => '0.8'
                            );

                        }

                    }

                }

            } elseif (class_exists('woocommerce')) {

                $wc_options = get_option('woocommerce_permalinks');
                $whitelisted_requests[] = $wc_options['product_base'] . '/';
                $cart_page_id = wc_get_page_id('cart');
                $parsed_url = parse_url($cart_page_id ? get_permalink($cart_page_id) : '');

                if (isset($parsed_url['path'])) {

                    $whitelisted_requests[] = $parsed_url['path'];

                }

            }

            if (!isset($options['allow_all_content'])) {

                $all_terms = get_terms(array(
                    'hide_empty' => false
                ));

                foreach ($all_terms as $term) {

                    if (taxonomy_exists($term->taxonomy)) {

                        $detected_urls[] = get_term_link($term);
                        $detected_urls[] = get_term_link($term) . 'page/([0-9]%2B)/';

                        if (isset($options['enable_sitemap']) && $options['enable_sitemap']) {

                            $term_latest_post = get_posts(array(
                                'tax_query' => array(
                                    array(
                                        'taxonomy' => $term->taxonomy,
                                        'field' => 'term_id',
                                        'terms' => $term->term_id,
                                    )
                                ),
                                'post_type'      => 'any',
                                'posts_per_page' => '1',
                                'orderby'        => 'date',
                                'order'          => 'DESC'
                            ));

                            if ($term_latest_post) {

                                if ($term->taxonomy == 'post_tag') {

                                    $sitemap_urls[] = array(
                                        'loc' => get_term_link($term),
                                        'lastmod' => get_the_modified_date('Y-m-d',$term_latest_post[0]),
                                        'changefreq' => 'weekly',
                                        'priority' => '0.5'
                                    );

                                } else {

                                    $sitemap_urls[] = array(
                                        'loc' => get_term_link($term),
                                        'lastmod' => get_the_modified_date('Y-m-d',$term_latest_post[0]),
                                        'changefreq' => 'daily',
                                        'priority' => '0.8'
                                    );

                                }

                            }

                        }

                    }

                }

            }

    		if (class_exists('woocommerce') ) {

                global $woocommerce;

                $checkout_page_url = function_exists('wc_get_cart_url') ? wc_get_checkout_url() : $woocommerce->cart->get_checkout_url();
                $detected_urls[] = $checkout_page_url . 'order-received/';
                $detected_urls[] = $checkout_page_url . 'order-received/' . urlencode('([0-9]+)') . '/';
                $detected_urls[] = $checkout_page_url . 'add-payment-method/';
                $detected_urls[] = $checkout_page_url . 'delete-payment-method/';
                $detected_urls[] = $checkout_page_url . 'set-default-payment-method/';

                $my_account_page_id = wc_get_page_id('myaccount');

                if ($my_account_page_id && get_permalink($my_account_page_id)) {

                    $my_account_page_url =  get_permalink($my_account_page_id);
                    $detected_urls[] = $my_account_page_url . 'orders/';
                    $detected_urls[] = $my_account_page_url . 'downloads/';
                    $detected_urls[] = $my_account_page_url . 'edit-account/';
                    $detected_urls[] = $my_account_page_url . 'addresses/';
                    $detected_urls[] = $my_account_page_url . 'payment-methods/';
                    $detected_urls[] = $my_account_page_url . 'lost-password/';
                    $detected_urls[] = $my_account_page_url . 'customer-logout/';

                }

                $whitelisted_requests[] = '/wc-api/(.*)/';

    		}

            if (!empty($detected_urls)) {

                foreach($detected_urls as $key => $url) {

                    $parsed_url = parse_url($url);

                    if (isset($parsed_url['port'])) {

                        $reconstructed_host = $parsed_url['host'] . ':' . $parsed_url['port'];

                    } else {

                        $reconstructed_host = $parsed_url['host'];

                    }

                    if ($reconstructed_host != $_SERVER['HTTP_HOST']) {

                        unset($detected_urls[$key]);

                    } else {

                        $detected_urls[$key] = $parsed_url['path'];

                    }

                }

            }

            if (get_option('bwpl_slug')) {

                $detected_urls[] = '/' . get_option('bwpl_slug');
                $detected_urls[] = '/' . get_option('bwpl_slug') . '/';
                $detected_urls[] = '/' . get_option('bwpl_slug') . '-wp-login.php';

        	}

            if (!empty($detected_urls) || isset($options['allow_all_content'])) {

                $detected_urls = array_unique($detected_urls);
                $detected_urls = array_diff($detected_urls, $this->black_list);
                sort($detected_urls);

                $sitemap_disabled = false;

                if (defined('WPSEO_VERSION')) {

                    $yoast_options = get_option('wpseo');

                    if (isset($yoast_options['enable_xml_sitemap']) && $yoast_options['enable_xml_sitemap']) {

                        $sitemap_disabled = true;

                    }

                }

    			if (!$sitemap_disabled && isset($sitemap_urls) && $sitemap_urls && isset($options['enable_sitemap']) && $options['enable_sitemap'] && 
    			((!file_exists(dafCommon::get_home_path() . 'sitemap.xml') && is_writable(dafCommon::get_home_path())) || is_writable(dafCommon::get_home_path() . 'sitemap.xml')) && 
    			((!file_exists(dafCommon::get_home_path() . 'robots.txt') && is_writable(dafCommon::get_home_path())) || is_writable(dafCommon::get_home_path() . 'robots.txt'))) {

                    $sitemap = '<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="' . site_url('wp-sitemap.xsl') . '" ?>
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
    <url>
        <loc>' . htmlspecialchars(site_url()) . '</loc>
        <lastmod>' . date('Y-m-d') . '</lastmod>
        <changefreq>daily</changefreq>
        <priority>1.0</priority>
    </url>
';

                    foreach ($sitemap_urls as $sitemap_url) {

                        $sitemap .= '    <url>
        <loc>' . htmlspecialchars($sitemap_url['loc']) . '</loc>
        <lastmod>' . $sitemap_url['lastmod'] . '</lastmod>
        <changefreq>' . $sitemap_url['changefreq'] . '</changefreq>
        <priority>' . $sitemap_url['priority'] . '</priority>
    </url>
';

                    }

                    $sitemap .= '</urlset>
';
    				$f = @fopen(dafCommon::get_home_path() . 'sitemap.xml', 'w');
    				fwrite($f, $sitemap);
                    $robots = 'User-agent: *
Disallow: /wp-admin/
Allow: /wp-admin/admin-ajax.php
Sitemap: ' . site_url('sitemap.xml') . '
';
        			$f = @fopen(dafCommon::get_home_path() . 'robots.txt', 'w');
    				fwrite($f, $robots);

    			} else {

                    $this->daf_remove_sitemap();

    			}

                $themes[] = get_stylesheet();
                $themes[] = get_template();
                $themes = array_unique($themes);

                $htaccess = "# BEGIN Deny All Firewall
<IfModule mod_rewrite.c>
RewriteEngine on
RewriteBase /

";

                if (isset($options['force_ssl']) && $options['force_ssl']) {

                    $htaccess .= "# Force SSL
RewriteCond %{HTTPS} off
RewriteRule (.*) https://%{SERVER_NAME}/$1 [R,L]

";

                }

                $htaccess .= "# Block requests with no HTTP_HOST
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . "$\" [NC]
RewriteCond %{HTTP_HOST} \"!^([a-z0-9])\"
RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

";

                if ($external_ip && isset($options['enable_lock_down']) && $options['enable_lock_down']) {

                    $htaccess .= "# Strip Query Strings to prevent SQL injection etc
RewriteCond %{QUERY_STRING} !^$
RewriteCond %{QUERY_STRING} \"!^ga_action=\"
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!\.(css|ico|js|png|ttf|woff|woff2)$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/load-styles.php', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/load-scripts.php', 'relative')) . "$\" [NC]
RewriteCond %{THE_REQUEST} \"!^(POST|GET) /wp-json/\"
RewriteCond %{THE_REQUEST} \"!^(POST|GET) /wp-admin/admin-ajax.php\"
RewriteCond %{QUERY_STRING} \"!^post_type=([a-z]+)&p=([0-9]+)&preview=true$\"
";

            		if (class_exists('woocommerce') ) {

                        $htaccess .= "RewriteCond %{QUERY_STRING} \"!^wc-ajax=\"
RewriteCond %{QUERY_STRING} \"!^download_file=([0-9]+)&order=wc_order_([0-9a-zA-Z]+)&uid=([0-9a-z]+)&key=([0-9a-z]+)$\"
";

            		}

                    if (get_option('bwpl_slug')) {

                        $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url(get_option('bwpl_slug') . '-wp-login.php', 'relative')) . "$\"
";

                	} else {

                        $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-login.php', 'relative')) . "$\"
";

                	}

                    if (isset($options['allow_all_ips']) && $options['allow_all_ips']) {

                        $htaccess .= "RewriteCond %{REQUEST_URI} \"!^/wp-admin/\" [NC]
";

            		    if (class_exists('Vc_Manager') ) {

                            $htaccess .= "RewriteCond %{QUERY_STRING} \"!^vc_editable=true&vc_post_id=([0-9]+)&_vcnonce=([0-9a-z]+)$\"
";

                		}

                    } else {

                        foreach ($session_ips as $session_ip) {

                            if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                                $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                            } else {

                                $htaccess .= "RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                            }

                        }

                    }

                    if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                        $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"!^" . str_replace('.', '\.', $external_ip) . "\"
RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $external_ip) . "\"
";

                    } else {

                        $htaccess .= "RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $external_ip) . "\"
";

                    }

                    if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                        foreach ($whitelisted_requests as $whitelisted_request) {

                            $htaccess .= "RewriteCond %{THE_REQUEST} \"!^(POST|GET) " . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "\" [NC]
";

                        }

                    }

                    $htaccess .= "RewriteRule (.*) /$1? [R=301,L]

";

                    $htaccess .= "# Block POST requests to prevent file upload etc
RewriteCond %{REQUEST_METHOD} POST
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . "$\" [NC]
RewriteCond %{THE_REQUEST} \"!^POST /wp-json/\"
RewriteCond %{THE_REQUEST} \"!^POST /wp-admin/admin-ajax.php\"
";

                    if (get_option('bwpl_slug')) {

                        $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url(get_option('bwpl_slug') . '-wp-login.php', 'relative')) . "$\"
";

                	} else {

                        $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-login.php', 'relative')) . "$\"
";

                	}

                    if (isset($options['allow_all_ips']) && $options['allow_all_ips']) {

                        $htaccess .= "RewriteCond %{REQUEST_URI} \"!^/wp-admin/\" [NC]
";

                    } else {

                        foreach ($session_ips as $session_ip) {

                            if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                                $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                            } else {

                                $htaccess .= "RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                            }

                        }

                    }

                    if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                        $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"!^" . str_replace('.', '\.', $external_ip) . "\"
RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $external_ip) . "\"
";

                    } else {

                        $htaccess .= "RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $external_ip) . "\"
";

                    }

                    if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                        foreach ($whitelisted_requests as $whitelisted_request) {

                            $htaccess .= "RewriteCond %{THE_REQUEST} \"!^POST " . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "\" [NC]
";

                        }

                    }

                    $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

";

                }

                if (!empty($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $key => $whitelisted_request) {

                        if (substr($whitelisted_request, 0, 1) == '/') {

                            $whitelisted_requests[$key] = trim($whitelisted_request);

                        } else {

                            unset($whitelisted_requests[$key]);

                        }

                    }

                }

                if (!empty($whitelisted_redirects)) {

                    foreach ($whitelisted_redirects as $key => $whitelisted_redirect) {

                        if (substr($whitelisted_redirect['request'], 0, 1) == '/') {

                            $whitelisted_redirects[$key]['request'] = trim($whitelisted_redirect['request']);
                            $whitelisted_redirects[$key]['destination'] = trim($whitelisted_redirect['destination']);

                        } else {

                            unset($whitelisted_redirects[$key]);

                        }

                    }

                }

                if (!empty($whitelisted_redirects)) {

                    $htaccess .= "# 301 redirect whitelisted redirects
";

                    foreach ($whitelisted_redirects as $whitelisted_redirect) {

                        $htaccess .= "RewriteRule \"^" . str_replace('.', '\.', substr($whitelisted_redirect['request'], 1)) . "$\" \"" . str_replace('.', '\.', $whitelisted_redirect['destination']) . "\" [L,R=301]
";

                    }

                }

                $htaccess .= "# Only allow existing files from the theme directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/themes/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !-f
";

                if (!empty($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/themes/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow files from theme and / or child theme directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/themes/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-content/themes/', 'relative')) . "(.*)/screenshot\.png$\" [NC]
";

                foreach ($themes as $theme) {

                    $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-content/themes/', 'relative')) . "$theme/\" [NC]
";

                }

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/themes/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Allow css, fonts, images and js from the theme and / or child theme directory
";
                foreach ($themes as $theme) {

                    $htaccess .= "RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/themes/', 'relative')) . "$theme/\" [NC]
";

                }

                $htaccess .= "RewriteCond %{REQUEST_FILENAME} !\.(css|eot|gif|jpeg|jpg|js|png|svg|ttf|woff|woff2)$ [NC]
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/themes/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow existing files from the uploads directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/uploads/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !-f
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/uploads/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow css, documents, images, js, videos, pdf, svg and webmanifest from the uploads directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/uploads/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_URI} \"!\.(bmp|css|doc|docx|ico|jpeg|jpg|js|gif|m4a|mp4|mpg|png|pdf|svg|webmanifest)$\" [NC]
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/uploads/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow existing files from the plugins directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/plugins/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !-f
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/plugins/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow css, fonts, html, images, js and map from the plugins directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/plugins/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/remote-addr.php', 'relative')) . "$\" [NC]
";

                $htaccess .= "RewriteCond %{REQUEST_FILENAME} !\.(css|eot|gif|htm|html|jpeg|jpg|js|map|png|svg|ttf|woff||woff2)$ [NC]
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/plugins/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow existing files from the wp-includes directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-includes/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !-f
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-includes/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow css, fonts, gif, js, png, svg, ttf and woff from the wp-includes directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-includes/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-includes/js/tinymce/wp-tinymce.php', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_FILENAME} !\.(css|eot|gif|js|png|svg|ttf|woff|woff2)$ [NC]
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-includes/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow existing files or directories from the wp-admin directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-admin/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !-d
RewriteCond %{REQUEST_FILENAME} !-f
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-admin/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow css, images, php, png and js from the wp-admin directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-admin/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !\.(css|gif|svg|php|png|js)$ [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/', 'relative')) . "$\" [NC]
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-admin/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

";

                if (!isset($options['allow_all_ips']) && $external_ip) {

                    $htaccess .= "# Only allow css, js, svg, load-styles.php, load-scripts.php and index.php from the wp-admin directory unless logged in or local
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-admin/', 'relative')) . "\" [NC]
";

                    foreach ($session_ips as $session_ip) {

                        if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                            $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                        } else {

                            $htaccess .= "RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                        }

                    }

                    if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                        $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"!^" . str_replace('.', '\.', $external_ip) . "\"
RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $external_ip) . "\"
";

                    } else {

                        $htaccess .= "RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $external_ip) . "\"
";

                    }

                    $htaccess .= "RewriteCond %{REQUEST_FILENAME} !\.(css|js|svg)$ [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/load-styles.php', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/load-scripts.php', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/index.php', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-admin/admin-ajax.php', 'relative')) . "$\" [NC]
";

                    if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                        foreach ($whitelisted_requests as $whitelisted_request) {

                            if (preg_match('/^' . str_replace('/', '\/', site_url('wp-admin/', 'relative')) . '/', $whitelisted_request)) {

                                $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                            }

                        }

                    }

                    $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

";

                }

                if (!get_option('bwpl_slug')) {

                    $htaccess .= "# Allow users to login
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-login.php', 'relative'), '/')) . "$ - [L]

";

                } else {

                    $htaccess .= "# Allow users to unlock password protected posts and pages
RewriteCond %{QUERY_STRING} \"^action=postpass$\"
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-login.php', 'relative'), '/')) . "$ " . str_replace('.', '\.', site_url(get_option('bwpl_slug') . '-wp-login.php', 'relative')) . " [QSA,L]

# Allow confirm_admin_email
RewriteCond %{QUERY_STRING} \"&action=confirm_admin_email&wp_lang=\"
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-login.php', 'relative'), '/')) . "$ " . str_replace('.', '\.', site_url(get_option('bwpl_slug') . '-wp-login.php', 'relative')) . " [QSA,L]

";

                }

                if (isset($options['enable_sitemap']) && $options['enable_sitemap']) {

                    $htaccess .= "# Allow DAF generated sitemap.xml and robots.txt
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('robots.txt', 'relative'), '/')) . "$ - [L]
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('sitemap.xml', 'relative'), '/')) . "$ - [L]

";

                }

                $htaccess .= "# Allow WP-Cron
";

                if ($external_ip && isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                    $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"^" . str_replace('.', '\.', $external_ip) . "\" [OR]
RewriteCond %{REMOTE_ADDR} \"^" . str_replace('.', '\.', $external_ip) . "\"
";

                } elseif ($external_ip) {

                    $htaccess .= "RewriteCond %{REMOTE_ADDR} \"^" . str_replace('.', '\.', $external_ip) . "\"
";

                }

                $htaccess .= "RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-cron.php', 'relative'), '/')) . "$ - [L]

# Allow the main index.php
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('index.php', 'relative'), '/')) . "$ - [L]

# Allow directories already dealt with above
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-content/themes/', 'relative'), '/')) . " - [L]
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-content/uploads/', 'relative'), '/')) . " - [L]
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-content/plugins/', 'relative'), '/')) . " - [L]
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-admin/', 'relative'), '/')) . " - [L]
RewriteRule ^wp-includes/ - [L]

# Only allow existing files from the wp-content directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !-f
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Only allow images, css and js from the wp-content directory
RewriteCond %{REQUEST_URI} \"^" . str_replace('.', '\.', site_url('wp-content/', 'relative')) . "\" [NC]
RewriteCond %{REQUEST_FILENAME} !\.(css|gif|jpeg|jpg|js|png)$ [NC]
";

                if (isset($whitelisted_requests) && is_array($whitelisted_requests)) {

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        if (preg_match('/^' . str_replace('/', '\/', site_url('wp-content/', 'relative')) . '/', $whitelisted_request)) {

                            $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                        }

                    }

                }

                $htaccess .= "RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Allow wp-content already dealt with above
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('wp-content/', 'relative'), '/')) . " - [L]

# Allow Google Verification
RewriteRule ^" . str_replace('.', '\.', ltrim(site_url('google', 'relative'), '/')) . "[a-f0-9]{16}\.html$ - [L]

# Allow wp-json
RewriteCond %{REQUEST_URI} \"!^/wp-json/\"

";

                if (get_option('page_for_posts')) {

                    $parsed_url = parse_url(get_permalink(get_option('page_for_posts')));
                    $htaccess .= "# Allow pages of posts on the posts page
RewriteCond %{REQUEST_URI} \"!^" . $parsed_url['path'] . "page/([0-9]+)/$\"

";

                } else {

                    $htaccess .= "# Allow pages of posts on the homepage
RewriteCond %{REQUEST_URI} \"!^/page/([0-9]+)/$\"

";

                }

                if (isset($whitelisted_requests) && is_array($whitelisted_requests) && $whitelisted_requests) {

                    $htaccess .= "# Allow whitelisted requests
";

                    foreach ($whitelisted_requests as $whitelisted_request) {

                        $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . str_replace('(\.*)', '(.*)', str_replace('.', '\.', $whitelisted_request)) . "$\" [NC]
";

                    }

                $htaccess .= "
";

                }

                if (isset($options['enable_sitemap']) && $options['enable_sitemap']) {

                    $htaccess .= "# Allow DAF generated sitemap.xml and robots.txt
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-sitemap.xsl', 'relative')) . "$\" [NC]
";

                } else {

                    $htaccess .= "# Allow WordPress generated sitemap.xml and robots.txt
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('robots.txt', 'relative')) . "$\" [NC]
RewriteCond %{REQUEST_URI} \"!^" . str_replace('.', '\.', site_url('wp-sitemap', 'relative')) . "(.*)\.(xml|xsl)$\" [NC]

";

                }

                $htaccess .= "# Allow existing posts and terms
";

                if (isset($options['allow_all_content']) && $options['allow_all_content']) {

                    $htaccess .= "RewriteCond %{REQUEST_URI} \"!^\/(?:[a-z0-9]+[\-a-z0-9]*\/)*$\"
";

                }

                foreach($detected_urls as $url) {

                    $url = urldecode($url);

                    $htaccess .= "RewriteCond %{REQUEST_URI} \"!^$url$\"
";

                }

        		if (class_exists('woocommerce') ) {

                    $htaccess .= "RewriteCond %{REQUEST_URI} \"!^/wc-api/(.*)/\"
";

        		}

                if (isset($checkout_page_url) && $checkout_page_url) {

                    $parsed_url = parse_url($checkout_page_url);
                    $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . $parsed_url['path'] . "order-pay/\"
";

                }

                if (isset($my_account_page_url) && $my_account_page_url) {

                    $parsed_url = parse_url($my_account_page_url);
                    $htaccess .= "RewriteCond %{REQUEST_URI} \"!^" . $parsed_url['path'] . "view-order/\"
";

                }

                $htaccess .= "
RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]

# Block user sniffing
RewriteCond %{QUERY_STRING} \"^author=\" [OR,NC]
RewriteCond %{QUERY_STRING} \"&author=\" [NC]
RewriteRule .* " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]
";

                if (!isset($options['allow_all_ips'])) {

                    foreach ($session_ips as $session_ip) {

                        if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {

                            $htaccess .= "RewriteCond %{HTTP:CF-CONNECTING-IP} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                        } else {

                            $htaccess .= "RewriteCond %{REMOTE_ADDR} \"!^" . str_replace('.', '\.', $session_ip) . "$\"
";

                        }

                    }

                    $htaccess .= "RewriteCond %{REQUEST_URI} \"^/wp-json/wp/v2/users\" [NC]
RewriteRule . " . str_replace('.', '\.', site_url('wp-content/plugins/deny-all-firewall/403.php', 'relative')) . " [L]
";

                }

                $htaccess .= "
</IfModule>
# END Deny All Firewall
";

                return $htaccess;

            } else {

                return false;

            }

		}

		function daf_inject_rules($htaccess = false) {

			if ($htaccess && is_writable(dafCommon::get_home_path() . '.htaccess')) {

				$htaccess = $htaccess . "\n" . implode('', file(dafCommon::get_home_path() . '.htaccess'));

				$f = @fopen(dafCommon::get_home_path() . '.htaccess', 'w');
				fwrite($f, $htaccess);

                delete_option('daf_content_changed');

                return true;

			}

            return false;

		}

        function daf_refresh_htaccess_on_login($user_login, $user) {

            $options = get_option('daf_options');

            if (!isset($options['allow_all_ips']) && isset($options['enable_firewall']) && $options['enable_firewall']) {

    			$current_htaccess = file_get_contents(dafCommon::get_home_path() . '.htaccess');

                $current_ip_in_htaccess = true;

                if (isset($_SERVER['HTTP_CF_CONNECTING_IP']) && strpos($current_htaccess, '^' . str_replace('.', '\.', $_SERVER['HTTP_CF_CONNECTING_IP']) . '$') === false) {

                    $current_ip_in_htaccess = false;

                } elseif (isset($_SERVER['REMOTE_ADDR']) && strpos($current_htaccess, '^' . str_replace('.', '\.', $_SERVER['REMOTE_ADDR']) . '$') === false) {

                    $current_ip_in_htaccess = false;

                }

                if (strpos($current_htaccess, '# BEGIN Deny All Firewall') !== false && $current_ip_in_htaccess === false) {

                    $htaccess = $this->daf_create_htaccess($user->ID);

                    if ($htaccess) {

                        if ($this->daf_remove_rules()) {

                            $this->daf_inject_rules($htaccess);

                        }

                    }

                }

            }

        }

        function daf_refresh_htaccess_on_update_options($old_options, $options) {

            if (isset($options['enable_firewall']) && $options['enable_firewall']) {

                delete_option('daf_external_ip');
                $htaccess = $this->daf_create_htaccess();

                if ($htaccess) {

                    if ($this->daf_remove_rules()) {

                        $this->daf_inject_rules($htaccess);

                    }

    			}

            }

        }

        function daf_refresh_htaccess_on_add_options($option) {

            $options = get_option($option);

            if (isset($options['enable_firewall']) && $options['enable_firewall']) {

                delete_option('daf_external_ip');
                $htaccess = $this->daf_create_htaccess();

                if ($htaccess) {

                    if ($this->daf_remove_rules()) {

                        $this->daf_inject_rules($htaccess);

                    }

    			}

            }

        }

        function daf_refresh_htaccess_hourly_if_changed_and_firewall_enabled() {

            $options = get_option('daf_options');

            if (isset($options['enable_firewall']) && $options['enable_firewall'] && get_option('daf_content_changed')) {

                $htaccess = $this->daf_create_htaccess();

                if ($htaccess) {

                    if ($this->daf_remove_rules()) {

                        $this->daf_inject_rules($htaccess);

                    }

    			}

            }

        }

        function daf_theme_options_metabox_setup() {

            $options = get_option('daf_options');

            if (!isset($options['allow_all_content'])) {

                add_action('add_meta_boxes', array($this, 'daf_add_theme_options_metabox'));

            }

        }

        function daf_add_theme_options_metabox() {

            add_meta_box('daf_meta_box', __('Firewall', 'deny-all-firewall'), array($this, 'daf_render_theme_options_metabox'), null, 'side', 'high');

        }

        function daf_render_theme_options_metabox($post) {

?>
<p>If you are publishing new content (new post, page, category, tag, image, etc), then you will need to refresh your firewall rules to allow it through before you can see it ...</p>
<p><a href="#" id="daf-editor-refresh-rules" title="<?= __('Refresh Firewall Rulees', 'deny-all-firewall'); ?>" class="button-primary">Refresh Firewall Rules</a></p>

<script>
    jQuery(document).on('click', '#daf-editor-refresh-rules', function() {
        if (confirm(<?= json_encode(__('Are you sure?', 'deny-all-firewall')); ?>)) {
            jQuery('#daf-editor-refresh-rules').after(' <img id="daf-refresh-loading" src="/wp-admin/images/loading.gif" style="vertical-align: bottom;" />');
            jQuery.ajax({
            	url: ajaxurl,
            	data: {
                	action: 'daf_refresh_rules',
                	_ajax_nonce: '<?= wp_create_nonce('daf-refresh-rules'); ?>'
            	},
            	success: function(result){
            	    if (result.success) {
                        jQuery('#daf-refresh-loading').remove();
            	    } else {
                        alert('Something went wrong, sorry!');
                        jQuery('#daf-refresh-loading').remove();
            	    }
                },
            	error: function(){
                    alert('Something went wrong, sorry!');
                    jQuery('#daf-refresh-loading').remove();
                }
        	});
        }
    });
</script>
<?php

        }

        public function daf_deactivation() {

            $options = get_option('daf_options');
            $this->daf_remove_rules();

            if (isset($options['enable_sitemap']) && $options['enable_sitemap']) {

                $this->daf_remove_sitemap();

            }

            if (isset($options['enable_firewall'])) {

                unset($options['enable_firewall']);

            }

            update_option('daf_options', $options);

        }

        public function daf_site_status_tests($tests) {

            // Disable the loopback test if wp-login.php has been blocked as this will cause a false negative

            if (get_option('bwpl_slug') && isset($tests['async']['loopback_requests'])) {

                unset($tests['async']['loopback_requests']);

            }

            return $tests;

        }

	}

    if (!class_exists('dafCommon')) {

        require_once(dirname(__FILE__) . '/includes/class-daf-common.php');

    }

	$daf = new daf_class();

}

?>
